import json
import re

import requests
from django.conf import settings
from django.db import models, transaction
from django.utils.html import format_html
from django.utils.module_loading import import_string
from django.db.models.signals import post_save
from django.dispatch import receiver

from .user_list import NotifyUserList
from .category import NotifyCategory
from .choices import TYPE, STATE
from .file import NotifyFile
from .log import NotifyErrorLog
from .template import NotifyTemplate
from ..mixins import UserNotifyMixin
from ..utils.send_data import url_dict_call, operator_call, response_check, notify_create, system_notify_create
from ..clients import SMSClient, EmailClient, CallClient, TelegramClient, ViberClient, PushClient, WhatsAppClient

NotifyMixin = import_string(settings.GARPIX_NOTIFY_MIXIN)


class Notify(NotifyMixin, UserNotifyMixin):
    """
    Уведомление
    """
    subject = models.CharField(max_length=255, default='', blank=True, verbose_name='Тема')
    text = models.TextField(verbose_name='Текст')
    html = models.TextField(verbose_name='HTML', blank=True, default='')

    user = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True, on_delete=models.SET_NULL,
                             related_name='notifies', verbose_name='Пользователь (получатель)')
    email = models.EmailField(max_length=255, blank=True, null=True, verbose_name='Email Получателя',
                              help_text='Используется только в случае отсутствия указанного пользователя')
    sender_email = models.EmailField(max_length=255, blank=True, null=True, verbose_name='Email Отправителя')

    type = models.IntegerField(choices=TYPE.CHOICES, verbose_name='Тип')
    state = models.IntegerField(choices=STATE.CHOICES, default=STATE.WAIT, verbose_name='Состояние')
    category = models.ForeignKey(NotifyCategory, on_delete=models.CASCADE, related_name='notifies',
                                 verbose_name='Категория', blank=True, null=True)
    event = models.IntegerField(choices=settings.CHOICES_NOTIFY_EVENT, blank=True, null=True, verbose_name='Событие')
    files = models.ManyToManyField(NotifyFile, verbose_name='Файлы')

    is_read = models.BooleanField(default=False, verbose_name='Прочитано')
    data_json = models.TextField(blank=True, null=True, verbose_name='Данные пуш-уведомления (JSON)')
    room_name = models.CharField(max_length=255, null=True, blank=True, verbose_name='Название комнаты')

    users_list = models.ManyToManyField(NotifyUserList, blank=True, verbose_name='Списки пользователей для рассылки')

    created_at = models.DateTimeField(auto_now_add=True, verbose_name='Дата создания')
    send_at = models.DateTimeField(blank=True, null=True, verbose_name='Время начала отправки')
    sent_at = models.DateTimeField(blank=True, null=True, verbose_name='Дата отправки')

    def __str__(self):
        return self.subject

    def get_sender(self):
        pass

    def _send(self):
        if self.user:
            self.email = self.user.email
            self.phone = str(self.user.phone)
            self.telegram_chat_id = self.user.telegram_chat_id
            self.viber_chat_id = self.user.viber_chat_id
        elif self.email:
            self.email = self.email

        if self.phone is not None:
            self.phone = re.sub("[^0-9]", "", self.phone)

        if self.type == TYPE.EMAIL:
            EmailClient.send_email(self)
        if self.type == TYPE.SMS:
            SMSClient.send_sms(self)
        if self.type == TYPE.PUSH:
            PushClient.send_push(self)
        if self.type == TYPE.TELEGRAM:
            TelegramClient.send_telegram(self)
        if self.type == TYPE.VIBER:
            ViberClient.send_viber(self)
        if self.type == TYPE.CALL:
            CallClient.send_call(self)
        if self.type == TYPE.WHATSAPP:
            WhatsAppClient.send_whatsapp(self)

        self.save()

    def get_format_state(self):
        if self.state == STATE.WAIT:
            return format_html('<span style="color:orange;">В ожидании</span>')
        elif self.state == STATE.DELIVERED:
            return format_html('<span style="color:green;">Отправлено</span>')
        elif self.state == STATE.REJECTED:
            return format_html('<span style="color:red;">Отклонено</span>')
        elif self.state == STATE.DISABLED:
            return format_html('<span style="color:red;">Отправка запрещена</span>')
        return format_html('<span style="color:black;">Неизвестный статус</span>')

    get_format_state.short_description = 'Статус'

    def to_log(self, error_text):
        log = NotifyErrorLog(notify=self, error=error_text)
        log.save()

    @staticmethod
    def send(event, context, user=None, email=None, phone=None, files=None, data_json=None,  # noqa
             viber_chat_id=None, room_name=None, notify_templates=None, send_at=None, system=False, **kwargs):

        instance = None
        # Сначала забираем те данные, которые передали с методом
        notify_user = user if user else None
        notify_email = email if email else None
        notify_phone = phone if phone else None
        notify_viber_chat_id = viber_chat_id if viber_chat_id else None

        local_context = context.copy()

        user_want_message_check = None
        if hasattr(settings, 'NOTIFY_USER_WANT_MESSAGE_CHECK') and settings.NOTIFY_USER_WANT_MESSAGE_CHECK is not None:
            user_want_message_check = import_string(settings.NOTIFY_USER_WANT_MESSAGE_CHECK)

        data_json = json.dumps(data_json) if data_json is not None else None

        if system:
            if data_json is None:
                data_json = json.dumps(local_context) if local_context is not None else None

            data_dict = system_notify_create(context=data_json, user=notify_user,
                                             email=notify_email, phone=notify_phone, viber=notify_viber_chat_id,
                                             json=data_json, time=send_at, room=room_name, type_notify=TYPE.SYSTEM,
                                             event=event)
            instance = Notify.objects.create(**data_dict, **kwargs)
            return instance

        # Для выбора шаблонов из Категории или по ивенту
        if notify_templates:
            templates = NotifyTemplate.objects.filter(id__in=notify_templates, event=event, is_active=True)
        else:
            templates = NotifyTemplate.objects.filter(event=event, is_active=True)
        if files is None:
            files = []

        file_instances = []
        for f in files:
            instance = NotifyFile.objects.create(file=f)
            file_instances.append(instance)

        for template in templates:

            notify_users_lists = template.user_lists.all()

            # Формируем список основных получателей письма
            # Из шаблона, если методом не было передано данных
            template_user = template.user if template.user else None
            template_email = template.email if template.email else None
            template_phone = template.phone if template.phone else None
            template_viber_chat_id = template.viber_chat_id if template.viber_chat_id else None

            # Данные, которые указал пользователь у себя в профиле в приоритете
            if notify_user:
                notify_email = notify_user.email if notify_user.email else notify_email
                notify_phone = notify_user.phone if notify_user.phone else notify_phone
                notify_viber_chat_id = notify_user.viber_chat_id if notify_user.viber_chat_id else notify_viber_chat_id

            if notify_user is None and template_user:
                notify_user = template.user if template.user else None
                notify_email = template.user.email if template.user.email else notify_email
                notify_phone = template.user.phone if template.user.phone else notify_phone
                notify_viber_chat_id = template.user.viber_chat_id if template.user.viber_chat_id else notify_viber_chat_id

            if notify_email is None:
                notify_email = template_email

            if notify_phone is None:
                notify_phone = template_phone

            if notify_viber_chat_id is None:
                notify_viber_chat_id = template_viber_chat_id

            # Проверка, хочет ли получить сообщение
            if user_want_message_check is not None:  # noqa
                if not notify_users_lists.exists():
                    user_check = user_want_message_check(event, template.type, notify_user)
                    if user_check is None:
                        return None
                else:
                    # Если у нас шаблон со списками, то передаем в функцию и тут формируем новый из тех юзеов,
                    # которым сообщения нужны
                    # Если список пустой, то отменяем отправку
                    notify_users_lists = user_want_message_check(
                        event, template.type, notify_user, notify_users_lists)
                    if notify_users_lists is None:
                        return None
                # Добавляем пользователя в контекст, если его там не передали
            if notify_user is not None:
                if local_context is not None:
                    local_context.update({
                        'user': notify_user,
                    })
                else:
                    local_context = {
                        'user': notify_user,
                    }
            local_context['event_id'] = event

            # Проверка на наличие списка в шаблоне
            # Если в шаблоне передаются списки пользователей, то отдаем их уведомлениям
            # Если уведомление для одного пользователя, то создаем для одного
            # Также идет проверка на время оправки
            if send_at is not None:
                notify_send = send_at
            else:
                notify_send = template.send_at
            data_dict = notify_create(template=template, context=local_context, user=notify_user,
                                      email=notify_email, phone=notify_phone, viber=notify_viber_chat_id,
                                      json=data_json, time=notify_send, room=room_name)
            instance = Notify.objects.create(
                **data_dict,
                **kwargs
            )
            if notify_users_lists.exists():
                instance.users_list.add(*notify_users_lists)
            file_instance = instance.files
            for f in file_instances:
                file_instance.add(f)
            instance.save()

        return instance

    @staticmethod
    def call(phone, user=None, url=None, **kwargs):
        call_url_type = CallClient.CALL_URL_TYPE

        if user is not None:
            phone = user.phone if user.phone else phone
        # Для некоторых операторов из представленного списка, мы можем сгенерировать код на нашей стороне
        # Для этого можем передать в функцию кастомную ссылку и дополнительные параметры
        if url is not None:
            url = url
        else:
            url = url_dict_call[call_url_type]
        main_url = url.format(**operator_call[call_url_type], to=phone, **kwargs)
        response_url = requests.get(main_url)
        response_dict = response_url.json()
        value = CallClient.get_value_checker(response_dict)

        response = response_check(response=response_dict, operator_type=call_url_type, status=value)
        if value == "OK":
            return '{Code}'.format(**response)
        else:
            return None

    class Meta:
        verbose_name = 'Уведомление'
        verbose_name_plural = 'Уведомления'


@receiver(post_save, sender=Notify)
def system_post_save(sender, instance, created, **kwargs):
    from garpix_notify.tasks.tasks import send_system_notifications
    if created and instance.type == TYPE.SYSTEM:
        transaction.on_commit(lambda: send_system_notifications.delay(instance.pk))



from django.conf import settings

from garpix_notify.models.choices import CALL_URL, SMS_URL
from garpix_notify.models.config import NotifyConfig


try:
    config = NotifyConfig.get_solo()
    SMS_API_ID = config.sms_api_id
    SMS_LOGIN = config.sms_login
    SMS_PASSWORD = config.sms_password
    SMS_FROM = config.sms_from
except Exception:
    SMS_API_ID = getattr(settings, 'SMS_API_ID', 1234567890)
    SMS_LOGIN = getattr(settings, 'SMS_LOGIN', '')
    SMS_PASSWORD = getattr(settings, 'SMS_PASSWORD', '')
    SMS_FROM = getattr(settings, 'SMS_FROM', '')

try:
    config = NotifyConfig.get_solo()
    CALL_API_ID = config.call_api_id
    CALL_LOGIN = config.call_login
    CALL_PASSWORD = config.call_password
except Exception:
    CALL_API_ID = getattr(settings, 'CALL_API_ID', 1234567890)
    CALL_LOGIN = getattr(settings, 'CALL_LOGIN', '')
    CALL_PASSWORD = getattr(settings, 'CALL_PASSWORD', '')
    CALL_FROM = getattr(settings, 'CALL_FROM', '')

url_dict_call = {
    0: '{url}?phone={to}&api_id={api_id}&json=1',
    1: '{url}?phone={to}&login={login}&password={password}&json=1',
    2: '{url}?login={login}&psw={password}&phones={to}&mes=code&call=1&fmt=3',
    3: '{url}?phone={to}&key={password}&service_id={login}',
}

operator_call = {
    0: {
        'url': CALL_URL.SMSRU_CALL_URL,
        'api_id': CALL_API_ID},
    1: {
        'url': CALL_URL.SMSRU_CALL_URL,
        'login': CALL_LOGIN,
        'password': CALL_PASSWORD},
    2: {
        'url': CALL_URL.SMSCENTRE_URL,
        'login': CALL_LOGIN,
        'password': CALL_PASSWORD},
    3: {
        'url': CALL_URL.UCALLER_URL,
        'login': CALL_LOGIN,
        'password': CALL_PASSWORD}}

url_dict_sms = {
    0: '{url}?api_id={api_id}&to={to}&msg={text}&json=1',
    1: '{url}?login={user}&pwd={pwd}&phones={to}&message={text}&sender={from_text}',
    2: '{url}?login={user}&password={pwd}&phone={to}&text={text}',
    3: '{url}?login={user}&psw={pwd}&phones={to}&mes={text}',
    4: '{url}?login={user}&password={pwd}&txt={text}&to={to}',
    5: '{url}?method=push_msg&format=json&key={api_id}&text={text}&phone={to}&sender_name={from_text}',
    6: '{url}?user={user}&pwd={pwd}&sadr={from_text}&text={text}&dadr={to}',
}

operator_sms = {
    0: {
        'url': SMS_URL.SMSRU_URL,
        'api_id': CALL_API_ID},
    1: {
        'url': SMS_URL.INFOSMS_URL,
        'user': SMS_LOGIN,
        'pwd': SMS_PASSWORD,
        'from_text': SMS_FROM},
    2: {
        'url': SMS_URL.IQSMS_URL,
        'user': SMS_LOGIN,
        'pwd': SMS_PASSWORD},
    3: {
        'url': SMS_URL.SMSCENTRE_URL,
        'user': SMS_LOGIN,
        'pwd': SMS_PASSWORD},
    4: {
        'url': SMS_URL.SMS_SENDING_URL,
        'user': SMS_LOGIN,
        'pwd': SMS_PASSWORD},
    5: {
        'url': SMS_URL.SMS_PROSTO_URL,
        'user': SMS_LOGIN,
        'pwd': SMS_PASSWORD,
        'from_text': SMS_FROM},
    6: {
        'url': SMS_URL.WEBSZK_URL,
        'user': SMS_LOGIN,
        'pwd': SMS_PASSWORD,
        'from_text': SMS_FROM},
}


def response_check(response, operator_type, status):
    if status == "OK":
        response_processing = {
            0 or 1: {
                'Status': response.get('status'),
                'Code': response.get('code'),
                'Balance': response.get('balance'),
                'ID_Call': response.get('call_id')},

            2: {'Status': response.get('id'),
                'Code': response.get('code'),
                'ID_Call': response.get('cnt'),
                'Balance': response.get('balance')},

            3: {'Status': response.get('status'),
                'Code': response.get('code'),
                'Balance': response.get('balance'),
                'ID_Call': response.get('unique_request_id')}}
        return response_processing[operator_type]

    if status == "BAD":
        response_processing = {
            0 or 1: {
                'Status': response.get('status'),
                'Status_code': response.get('status_code'),
                'Status_text': response.get('status_text')},
            2: {'Status': response.get('status'),
                'Status_code': response.get('error_code'),
                'Status_text': response.get('error')},
            3: {'Status': response.get('status'),
                'Status_code': response.get('code'),
                'Status_text': response.get('error')}}
        return response_processing[operator_type]

    return None


def notify_create(template, context, user, email, phone, viber, json, time, room):
    data_dict = {
        'subject': template.render_subject(context),
        'text': template.render_text(context),
        'html': template.render_html(context),
        'user': user,
        'email': email,
        'phone': phone if phone is not None else "",
        'viber_chat_id': viber if viber is not None else "",
        'type': template.type,
        'event': template.event,
        'category': template.category if template.category else None,
        'data_json': json,
        'send_at': time,
        'room_name': room,
    }
    return data_dict


def system_notify_create(type_notify, context, user, email, phone, viber, json, time, room, event):
    data_dict = {
        'user': user,
        'text': context,
        'email': email,
        'phone': phone if phone is not None else "",
        'viber_chat_id': viber if viber is not None else "",
        'type': type_notify,
        'event': event,
        'category': None,
        'data_json': json if json else context,
        'send_at': time,
        'room_name': room,
    }
    return data_dict



import requests
from django.conf import settings
from django.utils.timezone import now

from garpix_notify.models.config import NotifyConfig
from garpix_notify.models.choices import STATE, CALL_URL
from garpix_notify.utils.send_data import url_dict_call, operator_call, response_check

try:
    config = NotifyConfig.get_solo()
    IS_CALL_ENABLED = config.is_call_enabled
    CALL_URL_TYPE = config.call_url_type
except Exception:
    IS_CALL_ENABLED = True
    CALL_URL_TYPE = getattr(settings, 'CALL_URL_TYPE', 0)


class CallClient:

    def __init__(self, notify):
        self.notify = notify

    CALL_URL_TYPE = CALL_URL_TYPE

    def _value_checker(self, response_dict):
        value = None
        if self.CALL_URL_TYPE in [CALL_URL.SMSRU_CALL_ID, CALL_URL.SMSRU_CALL_API_ID]:
            if response_dict['status'] == 'OK':
                value = "OK"
            else:
                value = "BAD"

        elif self.CALL_URL_TYPE == CALL_URL.SMSCENTRE_ID:
            if response_dict['error']:
                value = "BAD"
            else:
                value = "OK"

        elif self.CALL_URL_TYPE == CALL_URL.UCALLER_ID:
            if response_dict['status']:
                value = "OK"
            else:
                value = "BAD"
        return value

    def __send_call_code(self):  # noqa
        if not IS_CALL_ENABLED:
            self.notify.state = STATE.DISABLED
            return
        phone = f'{self.notify.phone}'

        try:
            url = url_dict_call[self.CALL_URL_TYPE].format(**operator_call[self.CALL_URL_TYPE], to=phone)
            response_url = requests.get(url)
            response_dict = response_url.json()
            value = self._value_checker(response_dict)

            response = response_check(response=response_dict, operator_type=self.CALL_URL_TYPE, status=value)

            self.notify.save_to_log(response=response, value=value)
        except Exception as e:
            self.notify.state = STATE.REJECTED
            self.notify.to_log(str(e))

    def save_to_log(self, response, value):
        if value == "OK":
            self.notify.to_log(
                'Status: {Status}, Code: {Code}, Balance: {Balance}, ID_Call: {ID_Call}'.format(**response))
            self.notify.state = STATE.DELIVERED
            self.notify.sent_at = now()
        elif value == "BAD":
            self.notify.to_log(
                'Status: {Status}, Status_code: {Status_code}, Status_text: {Status_text}'.format(**response))
            self.notify.state = STATE.REJECTED
        else:
            return None

    @classmethod
    def get_value_checker(cls, response_dict):
        return cls(notify=None)._value_checker(response_dict)

    @classmethod
    def get_url_type(cls):
        return cls.CALL_URL_TYPE

    @classmethod
    def send_call(cls, notify):
        cls(notify).__send_call_code()



import json

from django.utils.timezone import now
from garpix_notify.models.config import NotifyConfig
from garpix_notify.models.fcm import NotifyDevice
from garpix_notify.models.choices import STATE

try:
    config = NotifyConfig.get_solo()
    IS_PUSH_ENABLED = config.is_push_enabled
except Exception:
    IS_PUSH_ENABLED = True


class PushClient:

    def __init__(self, notify):
        self.notify = notify

    def __send_push_client(self):

        if not IS_PUSH_ENABLED:
            self.notify.state = STATE.DISABLED
            return

        if self.notify.user is None:
            self.notify.state = STATE.REJECTED
            self.notify.save()
            return

        devices = NotifyDevice.objects.filter(active=True, user=self.notify.user).distinct('device_id')

        data_json = {}
        if self.notify.data_json is not None:
            data_json = json.loads(self.notify.data_json)
        data_json.update({
            'notify_id': self.notify.id,
        })

        try:
            devices.send_message(title=self.notify.subject, body=self.notify.text, data=data_json)
            self.notify.state = STATE.DELIVERED
            self.notify.sent_at = now()
        except Exception as e:  # noqa
            self.notify.state = STATE.REJECTED
            self.notify.to_log(str(e))

    @classmethod
    def send_push(cls, notify):
        cls(notify).__send_push_client()



from django.contrib.auth import get_user_model
from django.db.models import Q


class ReceivingUsers:

    def __init__(self, users_list, value=None):
        self.users_list = users_list
        self.value = value

    def __returning_specific_list(self, receivers):
        # Убираем дубликаты пользователей для переданного значения из функции
        receivers_dict = list({v[self.value]: v for v in receivers}.values())
        receivers_new = [user[self.value] for user in receivers_dict]
        return receivers_new

    def __receiving_users(self):
        user_model = get_user_model()
        receivers = []
        for user_list in self.users_list:
            # Проверяем рассылку, не отмечена ли массовая рассылка
            if user_list.mail_to_all:
                users = user_model.objects.all()
                receivers.extend(
                    [{'user': user,
                      'email': user.email,
                      'phone': user.phone,
                      'viber_chat_id': user.viber_chat_id
                      } for user in users])

                if self.value:
                    return self.__returning_specific_list(receivers)
                return receivers

            # Собираем данные из дополнительного списка получателей, если он есть.
            users_participants = user_list.participants.all()
            if users_participants.exists():
                for participant in users_participants:
                    if participant.user or participant.email or participant.phone or participant.viber_chat_id:
                        receivers.append({
                            'user': participant.user,
                            'email': participant.user.email if participant.user else participant.email,
                            'phone': participant.user.phone if participant.user else participant.phone,
                            'viber_chat_id':
                                participant.user.viber_chat_id if participant.user else participant.viber_chat_id
                        })

            # Собираем данные из групп, которые входят в список рассылки
            group_users = user_model.objects.filter(groups__in=user_list.user_groups.all())

            if group_users.exists():
                receivers.extend(
                    [{'user': user,
                      'email': user.email,
                      'phone': user.phone,
                      'viber_chat_id': user.viber_chat_id
                      } for user in group_users])

            if self.value:
                return self.__returning_specific_list(receivers)

            return receivers

    @classmethod
    def run_receiving_users(cls, users_list, value=None):
        return cls(users_list, value).__receiving_users()
